/*
 * Standard containers, std::list, exercise 1.
 */
#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <algorithm>
#include <functional>
#include <iterator>
using namespace std;

template<typename T>
ostream& operator<<(ostream& os, const list<T>& v)
{
   copy(begin(v), end(v), ostream_iterator<T>(cout, " "));
   return os;
}

template<typename T>
struct by_length 
{
   bool operator()(const T& arg1, const T& arg2) const
   {
      return arg1.size() < arg2.size();
   }

   typedef bool result_type;
   typedef T    first_argument_type;
   typedef T    second_argument_type;
};

template<int size, typename T = std::string> 
struct longer_than 
{
   bool operator()(const T& arg) {
      return size < arg.size();
   }

   typedef bool result_type;
   typedef T    argument_type;
};

const list<string> remove_word {
      "a", "and", "as", "be", "but", "do", "for", "if", "in", "is", 
      "it", "its", "not", "of", "or", "that", "the", "to"
   };

int main(int argc, char* argv[])
{
   if (argc != 2)
   {
      cout << "usage: " << argv[0] << " <filename>\n";
      return 1;
   }

   ifstream input(argv[1]);

   if (! input)
   {
      cout << argv[1] << ": could not open.\n";
      return 2;
   }

   list<string> words;

   copy(istream_iterator<string>(input), istream_iterator<string>(), back_inserter(words));
   cout << "Words read:\n" << words << "\n\n";

   words.sort();
   cout << "After sorting:\n" << words << "\n\n";

   words.erase(unique(begin(words), end(words)), end(words));
   cout << "After removing duplicates:\n" << words << "\n\n";

   words.sort(by_length<string>());  // list::sort is stable
   cout << "After sorting by increasing length:\n" << words << "\n\n";

   int n = count_if(begin(words), end(words), longer_than<5>());
   cout << "The number of words longer than five letters is " << n << ".\n\n";

   for (const auto& word : remove_word)
   {
      words.erase(remove(begin(words), end(words), word), end(words));
   }
   cout << "After removing common words:\n" << words << "\n\n";

   return 0;
}
