/*
 * multiset-01.cc    Standard Library, std::multiset, exercise 1.
 */
#include <iostream>
#include <iomanip>
#include <iterator>
#include <list>
#include <random>
#include <set>
using namespace std;

template<typename T>
ostream& operator<<(ostream& os, const list<T>& container)
{
   unsigned n = 0;
   for (const auto& x : container)
   {
      os << setw(4) << x << ((++n % 20 == 0) ? '\n' : '\0');
   }
   return os;
}

template<typename T>
void multiset_sort(list<T>& container)
{
   multiset<T> ms(begin(container), end(container));
   container.clear();
   container.insert(end(container), begin(ms), end(ms));
}

int main()
{
   // random number engine, the source of random numbers
   std::mt19937 rng;
   // distribution
   std::uniform_int_distribution<unsigned> uint_dist10{ 1, 9 };

   list<int> container;

   for (int i = 0; i < 100; ++i)
   {
      // the engine is passed to the function call operator of the
      // distribution to generate the next value.
      container.push_back(uint_dist10(rng));
   }
   
   cout << "Fre sortering:\n";
   cout << container << '\n';

   multiset_sort(container);

   cout << "Efter sortering:\n";
   cout << container << '\n';
   
   return 0;
}
